@@ -1,5 +1,6 @@
 import os
 import ee
+from functools import reduce
 
 AREA_FIELD = 'areaKm2'
 AREA_PERCENT_FIELD = 'areaKm2_percent'
@@ -7,6 +8,9 @@ MAX_AREA_SIZE = int(os.getenv('MAX_AREA_SIZE', '5000'))
 GADM_COLLECTION_PREFIX = 'users/hestiaplatform/gadm36_'
 
 
+def _date_year(date: str): return int(date.split('-')[0])
+
+
 def _id_to_level(id: str): return id.count('.')
 
 
@@ -51,18 +55,45 @@ def data_from_raster(image: ee.Image, collection: ee.FeatureCollection, reducer:
     return data.getInfo() if return_info else data
 
 
-def data_from_raster_by_period(image: ee.ImageCollection, collection, reducer, reducer_regions, scale,
-                               band_name, start_date, end_date, return_info=True):
-    reducer_func = getattr(ee.Reducer, reducer)
-    reducer_regions_func = getattr(ee.Reducer, reducer_regions) if reducer_regions else reducer_func
+def _filter_by_date(image: ee.ImageCollection, year: int):
+    return image.filter(ee.Filter.calendarRange(year, year, 'year'))
+
+
+def data_from_raster_by_period(
+    image: ee.ImageCollection, collection, reducer, reducer_regions, scale, band_name, start_date, end_date,
+    return_info=True, reducer_years=None
+):
+    # filter collection between dates
     filteredCollection = image.select(band_name).filterDate(start_date, end_date)
-    # reduce the image collection to one image by taking an average/sum over the images within the given time period
-    data = filteredCollection.reduce(reducer_func()).reduceRegions(
-        collection=collection,
-        reducer=reducer_regions_func(),
-        scale=scale
-    )
-    return data.getInfo() if return_info else data
+
+    def _reduce(filteredImage: ee.ImageCollection, reducer_name: str = reducer) -> ee.FeatureCollection:
+        reducer_regions_name = reducer_regions or reducer
+        reducer_func = getattr(ee.Reducer, reducer_name)
+        reducer_regions_func = getattr(ee.Reducer, reducer_regions_name)
+        # reduce the image collection to one image by taking an average/sum over the images within the given time period
+        return filteredImage.reduce(reducer_func()).reduceRegions(
+            collection=collection,
+            reducer=reducer_regions_func(),
+            scale=scale
+        )
+
+    if reducer_years:
+        first_col = _reduce(_filter_by_date(filteredCollection, _date_year(start_date)))
+        collections = reduce(lambda prev, year: prev.merge(
+            _reduce(_filter_by_date(filteredCollection, year))
+        ), range(_date_year(start_date) + 1, _date_year(end_date) + 1), first_col)
+        # apply the reducer for years
+        reducer_years_func = getattr(ee.Reducer, reducer_years)
+        dictionary = collections.reduceColumns(
+            reducer=reducer_years_func(),
+            selectors=[reducer_years]
+        )
+        # set result on first collection to always return a FeatureCollection
+        image = first_col.map(lambda c: c.set(reducer_years, dictionary.get(reducer_years)))
+    else:
+        image = _reduce(filteredCollection)
+
+    return image.getInfo() if return_info else image
 
 
 def area_km2(geometry: ee.Geometry): return geometry.area().divide(1000 * 1000)
